@@ -51,7 +51,7 @@
 #ifndef QT_NO_SHORTCUT
 #include <QMenuBar>
 #endif
-#if !defined(QT_NO_PRINTER) && HAVE(QTPRINTSUPPORT)
+#if !defined(QT_NO_PRINTPREVIEWDIALOG) && HAVE(QTPRINTSUPPORT)
 #include <QPrintPreviewDialog>
 #endif
 #include <QSlider>
@@ -73,6 +73,11 @@
 #include <QtNetwork/QNetworkDiskCache>
 #endif
 
+struct HighlightedElement {
+    QWebElement m_element;
+    QString m_previousStyle;
+};
+
 const int gExitClickArea = 80;
 QVector<int> LauncherWindow::m_zoomLevels;
 
@@ -170,7 +175,10 @@ void LauncherWindow::initializeView()
         WebViewGraphicsBased* view = new WebViewGraphicsBased(splitter);
         m_view = view;
 #ifndef QT_NO_OPENGL
-        toggleQGLWidgetViewport(m_windowOptions.useQGLWidgetViewport);
+        if (!m_windowOptions.useQOpenGLWidgetViewport)
+            toggleQGLWidgetViewport(m_windowOptions.useQGLWidgetViewport);
+        if (!m_windowOptions.useQGLWidgetViewport)
+            toggleQOpenGLWidgetViewport(m_windowOptions.useQOpenGLWidgetViewport);
 #endif
         view->setPage(page());
 
@@ -217,6 +225,7 @@ void LauncherWindow::applyPrefs()
     settings->setAttribute(QWebSettings::TiledBackingStoreEnabled, m_windowOptions.useTiledBackingStore);
     settings->setAttribute(QWebSettings::FrameFlatteningEnabled, m_windowOptions.useFrameFlattening);
     settings->setAttribute(QWebSettings::WebGLEnabled, m_windowOptions.useWebGL);
+    m_windowOptions.useWebAudio = settings->testAttribute(QWebSettings::WebAudioEnabled);
 
     if (!isGraphicsBased())
         return;
@@ -312,6 +321,17 @@ void LauncherWindow::createChrome()
     QAction* toggleWebGL = toolsMenu->addAction("Toggle WebGL", this, SLOT(toggleWebGL(bool)));
     toggleWebGL->setCheckable(true);
     toggleWebGL->setChecked(settings->testAttribute(QWebSettings::WebGLEnabled));
+#if !ENABLE(WEBGL)
+    toggleWebGL->setEnabled(false);
+#endif
+
+    QAction* toggleWebAudio = toolsMenu->addAction("Toggle WebAudio", this, SLOT(toggleWebAudio(bool)));
+    toggleWebAudio->setCheckable(true);
+#if ENABLE(WEB_AUDIO)
+    toggleWebAudio->setChecked(m_windowOptions.useWebAudio);
+#else
+    toggleWebAudio->setEnabled(false);
+#endif
 
     QAction* spatialNavigationAction = toolsMenu->addAction("Toggle Spatial Navigation", this, SLOT(toggleSpatialNavigation(bool)));
     spatialNavigationAction->setCheckable(true);
@@ -350,6 +370,8 @@ void LauncherWindow::createChrome()
 
     toolsMenu->addAction("Select Elements...", this, SLOT(selectElements()));
 
+    toolsMenu->addAction("Clear selection", this, SLOT(clearSelection()));
+
     QAction* showInspectorAction = toolsMenu->addAction("Show Web Inspector", m_inspector, SLOT(setVisible(bool)), QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_I));
     showInspectorAction->setCheckable(true);
     showInspectorAction->connect(m_inspector, SIGNAL(visibleChanged(bool)), SLOT(setChecked(bool)));
@@ -360,8 +382,12 @@ void LauncherWindow::createChrome()
     QAction* toggleAcceleratedCompositing = graphicsViewMenu->addAction("Toggle Accelerated Compositing", this, SLOT(toggleAcceleratedCompositing(bool)));
     toggleAcceleratedCompositing->setCheckable(true);
     toggleAcceleratedCompositing->setChecked(settings->testAttribute(QWebSettings::AcceleratedCompositingEnabled));
-    toggleAcceleratedCompositing->setEnabled(isGraphicsBased());
-    toggleAcceleratedCompositing->connect(toggleGraphicsView, SIGNAL(toggled(bool)), SLOT(setEnabled(bool)));
+
+    QAction* toggleAccelerated2dCanvas = graphicsViewMenu->addAction("Toggle Accelerated 2D canvas", this, SLOT(toggleAccelerated2dCanvas(bool)));
+    toggleAccelerated2dCanvas->setCheckable(true);
+    toggleAccelerated2dCanvas->setEnabled(settings->testAttribute(QWebSettings::AcceleratedCompositingEnabled));
+    toggleAccelerated2dCanvas->setChecked(settings->testAttribute(QWebSettings::Accelerated2dCanvasEnabled));
+    toggleAccelerated2dCanvas->connect(toggleAcceleratedCompositing, SIGNAL(toggled(bool)), SLOT(setEnabled(bool)));
 
     QAction* toggleResizesToContents = graphicsViewMenu->addAction("Toggle Resizes To Contents Mode", this, SLOT(toggleResizesToContents(bool)));
     toggleResizesToContents->setCheckable(true);
@@ -381,6 +407,13 @@ void LauncherWindow::createChrome()
     toggleQGLWidgetViewport->setChecked(m_windowOptions.useQGLWidgetViewport);
     toggleQGLWidgetViewport->setEnabled(isGraphicsBased());
     toggleQGLWidgetViewport->connect(toggleGraphicsView, SIGNAL(toggled(bool)), SLOT(setEnabled(bool)));
+#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
+    QAction* toggleQOpenGLWidgetViewport = graphicsViewMenu->addAction("Toggle use of QOpenGLWidget Viewport", this, SLOT(toggleQOpenGLWidgetViewport(bool)));
+    toggleQOpenGLWidgetViewport->setCheckable(true);
+    toggleQOpenGLWidgetViewport->setChecked(m_windowOptions.useQOpenGLWidgetViewport);
+    toggleQOpenGLWidgetViewport->setEnabled(isGraphicsBased());
+    toggleQOpenGLWidgetViewport->connect(toggleGraphicsView, SIGNAL(toggled(bool)), SLOT(setEnabled(bool)));
+#endif
 #endif
 
     QMenu* viewportUpdateMenu = graphicsViewMenu->addMenu("Change Viewport Update Mode");
@@ -477,6 +510,10 @@ void LauncherWindow::createChrome()
 #endif
     toggleScrollAnimator->setChecked(false);
 
+    QAction* toggleJavaScriptEnabled = settingsMenu->addAction("Enable Javascript", this, SLOT(toggleJavaScriptEnabled(bool)));
+    toggleJavaScriptEnabled->setCheckable(true);
+    toggleJavaScriptEnabled->setChecked(settings->testAttribute(QWebSettings::JavascriptEnabled));
+
     QAction* toggleInterruptingJavaScripteEnabled = settingsMenu->addAction("Enable interrupting js scripts", this, SLOT(toggleInterruptingJavaScriptEnabled(bool)));
     toggleInterruptingJavaScripteEnabled->setCheckable(true);
     toggleInterruptingJavaScripteEnabled->setChecked(false);
@@ -735,7 +772,7 @@ void LauncherWindow::toggleZoomTextOnly(bool b)
 
 void LauncherWindow::print()
 {
-#if !defined(QT_NO_PRINTER) && HAVE(QTPRINTSUPPORT)
+#if !defined(QT_NO_PRINTPREVIEWDIALOG) && HAVE(QTPRINTSUPPORT)
     QPrintPreviewDialog dlg(this);
     connect(&dlg, SIGNAL(paintRequested(QPrinter*)),
             page()->mainFrame(), SLOT(print(QPrinter*)));
@@ -745,7 +782,7 @@ void LauncherWindow::print()
 
 void LauncherWindow::screenshot()
 {
-    QPixmap pixmap = QPixmap::grabWidget(m_view);
+    QPixmap pixmap = m_view->grab();
     QLabel* label = 0;
     label = new QLabel;
     label->setAttribute(Qt::WA_DeleteOnClose);
@@ -763,7 +800,10 @@ void LauncherWindow::screenshot()
 #endif
 
 #ifndef QT_NO_OPENGL
-    toggleQGLWidgetViewport(m_windowOptions.useQGLWidgetViewport);
+    if (!m_windowOptions.useQOpenGLWidgetViewport)
+        toggleQGLWidgetViewport(m_windowOptions.useQGLWidgetViewport);
+    if (!m_windowOptions.useQGLWidgetViewport)
+        toggleQOpenGLWidgetViewport(m_windowOptions.useQOpenGLWidgetViewport);
 #endif
 }
 
@@ -800,14 +840,25 @@ void LauncherWindow::selectElements()
                                         QLineEdit::Normal, "a", &ok);
 
     if (ok && !str.isEmpty()) {
+        clearSelection();
         QWebElementCollection result =  page()->mainFrame()->findAllElements(str);
-        foreach (QWebElement e, result)
+        foreach (QWebElement e, result) {
+            HighlightedElement el = { e, e.styleProperty("background-color", QWebElement::InlineStyle) };
+            m_highlightedElements.append(el);
             e.setStyleProperty("background-color", "yellow");
+        }
         statusBar()->showMessage(QString("%1 element(s) selected").arg(result.count()), 5000);
     }
 #endif
 }
 
+void LauncherWindow::clearSelection()
+{
+    for (int i = 0; i < m_highlightedElements.size(); ++i)
+        m_highlightedElements[i].m_element.setStyleProperty("background-color", m_highlightedElements[i].m_previousStyle);
+    m_highlightedElements.clear();
+}
+
 void LauncherWindow::setDiskCache(bool enable)
 {
 #if !defined(QT_NO_NETWORKDISKCACHE) && !defined(QT_NO_DESKTOPSERVICES)
@@ -843,6 +894,11 @@ void LauncherWindow::toggleAcceleratedCompositing(bool toggle)
     page()->settings()->setAttribute(QWebSettings::AcceleratedCompositingEnabled, toggle);
 }
 
+void LauncherWindow::toggleAccelerated2dCanvas(bool toggle)
+{
+    page()->settings()->setAttribute(QWebSettings::Accelerated2dCanvasEnabled, toggle);
+}
+
 void LauncherWindow::toggleTiledBackingStore(bool toggle)
 {
     page()->settings()->setAttribute(QWebSettings::TiledBackingStoreEnabled, toggle);
@@ -860,6 +916,12 @@ void LauncherWindow::toggleWebGL(bool toggle)
     page()->settings()->setAttribute(QWebSettings::WebGLEnabled, toggle);
 }
 
+void LauncherWindow::toggleWebAudio(bool toggle)
+{
+    m_windowOptions.useWebAudio = toggle;
+    page()->settings()->setAttribute(QWebSettings::WebAudioEnabled, toggle);
+}
+
 void LauncherWindow::animatedFlip()
 {
     qobject_cast<WebViewGraphicsBased*>(m_view)->animatedFlip();
@@ -887,6 +949,11 @@ void LauncherWindow::toggleFrameFlattening(bool toggle)
     page()->settings()->setAttribute(QWebSettings::FrameFlatteningEnabled, toggle);
 }
 
+void LauncherWindow::toggleJavaScriptEnabled(bool enable)
+{
+    page()->settings()->setAttribute(QWebSettings::JavascriptEnabled, enable);
+}
+
 void LauncherWindow::toggleInterruptingJavaScriptEnabled(bool enable)
 {
     page()->setInterruptingJavaScriptEnabled(enable);
@@ -923,11 +990,28 @@ void LauncherWindow::toggleQGLWidgetViewport(bool enable)
     if (!isGraphicsBased())
         return;
 
+    if (enable)
+        m_windowOptions.useQOpenGLWidgetViewport = false;
     m_windowOptions.useQGLWidgetViewport = enable;
-    WebViewGraphicsBased* view = static_cast<WebViewGraphicsBased*>(m_view);
 
+    WebViewGraphicsBased* view = static_cast<WebViewGraphicsBased*>(m_view);
     view->setViewport(enable ? new QGLWidget() : 0);
 }
+
+void LauncherWindow::toggleQOpenGLWidgetViewport(bool enable)
+{
+    if (!isGraphicsBased())
+        return;
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
+    if (enable)
+        m_windowOptions.useQGLWidgetViewport = false;
+    m_windowOptions.useQOpenGLWidgetViewport = enable;
+
+    WebViewGraphicsBased* view = static_cast<WebViewGraphicsBased*>(m_view);
+    view->setViewport(enable ? new QOpenGLWidget() : 0);
+#endif
+}
 #endif
 
 void LauncherWindow::changeViewportUpdateMode(int mode)
